(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.vueComponents = {}));
}(this, (function (exports) { 'use strict';

  class Keys {
  }
  Keys.WORKFLOW = 'WORKFLOW';
  Keys.WORK = 'WORK';
  Keys.WORK_CORE = 'WORK_CORE';
  Keys.WORK_ERROR = 'WORK_ERROR';
  Keys.WORK_MESSAGE = 'WORK_MESSAGE';

  class Work {
      constructor(option, workflow, useLogger) {
          this._origin = {};
          this._inValues = [];
          this._askTimer = 0;
          this._option = option;
          this._workflow = workflow;
          this._logger = useLogger(Keys.WORK);
          if (option.pipe) {
              this._pipe = new WorkFlow(option.pipe, workflow.elements, useLogger);
          }
          option.params.forEach((param) => {
              this._origin[param.field] = param.value;
          });
      }
      get id() {
          return this._option.id;
      }
      get logger() {
          return this._logger;
      }
      get inputs() {
          return this._option.inputs.split('|').filter((e, index, arr) => arr.length > 1 ? 1 : e.length);
      }
      get outputs() {
          return this._option.outputs.split('|').filter((e) => e.length);
      }
      get(key) {
          return this._option[key];
      }
      set(key, value) {
          this._option[key] = value;
          return this._option[key];
      }
      start(data) {
          this._workflow.current = this;
          this._outValue = data;
          this._workflow.logger.push(this.logger);
          return this.doPull();
      }
      doPull() {
          this.logger.start(this.name);
          this.logger.add(Keys.WORK_CORE, this.name, '__DO_PULL__');
          this._inValues = [];
          this.inputs.forEach((input) => {
              if (input) {
                  const work = this._workflow.search(input);
                  if (work) {
                      this.logger.push(work.logger);
                      this._inValues.push(work.getValue(this._askTimer >= work._askTimer));
                  }
                  else {
                      this._inValues.push(null);
                  }
              }
              else {
                  this._inValues.push(null);
              }
          });
          return this.doProcess(...this._inValues);
      }
      doProcess(...args) {
          this._askTimer++;
          this.logger.add(Keys.WORK_CORE, this.name, '__DO_PROCESS__', this._outValue);
          return this._outValue;
      }
      getValue(isNew) {
          if (isNew) {
              this.doPull();
          }
          return this._outValue;
      }
      get name() {
          return this.constructor.name + `[${this.id}]`;
      }
  }
  Work.NAME = 'Worker';

  class WorkBlock extends Work {
      doProcess(any) {
          this._outValue = this._pipe?.start(any);
          return super.doProcess();
      }
  }
  WorkBlock.NAME = 'Block';

  class WorkCase extends Work {
      doProcess(cases, key) {
          let keys = key.split('|');
          for (let k of keys) {
              cases[k] = this._pipe;
          }
          this._outValue = cases;
          return super.doProcess();
      }
  }
  WorkCase.NAME = 'Case';

  class WorkDelay extends Work {
      doProcess(time, extra) {
          setTimeout(() => {
              this._pipe?.start(extra);
          }, time);
          return super.doProcess();
      }
  }
  WorkDelay.NAME = 'Delay';

  class WorkIfElse extends Work {
      doPull() {
          this.logger.add(Keys.WORK_CORE, this.name, '__DO_PULL__');
          return this.doProcess();
      }
      doProcess() {
          const work = this._workflow.search(this.inputs[0]);
          if (work) {
              const boolean = work.getValue(this._askTimer >= work._askTimer);
              if (boolean) {
                  const tp = this._workflow.search(this.inputs[1]);
                  if (tp) {
                      this._outValue = tp.getValue(this._askTimer >= tp._askTimer);
                  }
              }
              else {
                  const fp = this._workflow.search(this.inputs[2]);
                  if (fp) {
                      this._outValue = fp.getValue(this._askTimer >= fp._askTimer);
                  }
              }
          }
          else {
              this._outValue = null;
          }
          return super.doProcess();
      }
  }
  WorkIfElse.NAME = 'IfElse';

  class WorkSwitch extends Work {
      doProcess(input, cases, extra) {
          if (cases[input]) {
              this.logger.add(Keys.WORK_CORE, this.name, '__DO_PROCESS__', input);
              cases[input].start({ extra });
          }
          else if (cases['default']) {
              this.logger.add(Keys.WORK_CORE, this.name, '__DO_PROCESS__', 'default');
              cases['default'].start({ extra });
          }
          else {
              this.logger.add(Keys.WORK_CORE, this.name, '__DO_PROCESS__', 'no catch ' + input);
          }
          return super.doProcess();
      }
  }
  WorkSwitch.NAME = 'Switch';

  class WorkArrayConcat extends Work {
      doProcess(first, second) {
          this._outValue = first.concat(second);
          return super.doProcess(first, second);
      }
  }
  WorkArrayConcat.NAME = 'ArrayConcat';

  class WorkArrayFieldMerge extends Work {
      doProcess(arrayA, arrayB, keys) {
          this._outValue = [];
          arrayA.forEach(item => {
              let ext = arrayB.filter(sample => {
                  for (let key of keys) {
                      if (item[key] !== sample[key]) {
                          return false;
                      }
                  }
                  return true;
              });
              if (ext.length) {
                  item = Object.assign(item, ext[0]);
              }
              this._outValue.push(item);
          });
          return super.doProcess(arrayA, arrayB, keys);
      }
  }
  WorkArrayFieldMerge.NAME = 'ArrayMergeByField';

  class WorkArrayRemove extends Work {
      doProcess(array, any) {
          let index = array.indexOf(any);
          this._outValue = array.splice(index, 1);
          return super.doProcess(array, any);
      }
  }
  WorkArrayRemove.NAME = 'ArrayRemove';

  class WorkArraySum extends Work {
      doProcess(array) {
          this._outValue = array.reduce((previousValue, currentValue) => {
          });
          return super.doProcess(array);
      }
  }
  WorkArraySum.NAME = 'ArraySum';

  class WorkArrayFieldSum extends Work {
      doProcess(array, field, rate) {
          this._outValue = array.reduce((previousValue, currentValue) => {
              previousValue += currentValue[field] * (rate ? (currentValue[rate] ? currentValue[rate] : 1) : 1);
          });
          return super.doProcess(array, field, rate);
      }
  }
  WorkArrayFieldSum.NAME = 'ArraySumByField';

  class WorkArrayFilter extends Work {
      doProcess(array, extra) {
          this._outValue = array.filter((value, index, array) => {
              return this._pipe?.start({ value, index, array, extra })[0];
          });
          return super.doProcess(array, extra);
      }
  }
  WorkArrayFilter.NAME = 'ArrayFilter';

  class WorkArrayFlat extends Work {
      doProcess(array, depth) {
          this._outValue = array.flat(depth != -1 ? depth : Infinity);
          return super.doProcess(array, depth);
      }
  }
  WorkArrayFlat.NAME = 'ArrayFlat';

  class WorkArrayForEach extends Work {
      doProcess(array, extra) {
          this._outValue = array.forEach((item, index, array) => {
              this._pipe?.start({ item, index, array, extra });
          });
          return super.doProcess(array, extra);
      }
  }
  WorkArrayForEach.NAME = 'ArrayForEach';

  class WorkArrayIndexOf extends Work {
      doProcess(array, any, start) {
          this._outValue = array.indexOf(any, start);
          return super.doProcess(array, any, start);
      }
  }
  WorkArrayIndexOf.NAME = 'ArrayIndexOf';

  class WorkArrayJoin extends Work {
      doProcess(array, separator) {
          this._outValue = array.join(separator);
          return super.doProcess(array, separator);
      }
  }
  WorkArrayJoin.NAME = 'ArrayJoin';

  class WorkArrayLastIndexOf extends Work {
      doProcess(array, any, start) {
          this._outValue = array.lastIndexOf(any, start);
          return super.doProcess(array, any, start);
      }
  }
  WorkArrayLastIndexOf.NAME = 'ArrayLastIndexOf';

  class WorkArrayLength extends Work {
      doProcess(array) {
          this._outValue = array.length;
          return super.doProcess(array);
      }
  }
  WorkArrayLength.NAME = 'ArrayLength';

  class WorkArrayMap extends Work {
      doProcess(array, extra) {
          this._outValue = array.map((item) => {
              return this._pipe?.start({ item, extra })[0];
          });
          return super.doProcess(array, extra);
      }
  }
  WorkArrayMap.NAME = 'ArrayMap';

  class WorkArrayPop extends Work {
      doProcess(array) {
          this._outValue = array.pop();
          return super.doProcess(array);
      }
  }
  WorkArrayPop.NAME = 'ArrayPop';

  class WorkArrayPush extends Work {
      doProcess(array, any) {
          array.push(any);
          this._outValue = array;
          return super.doProcess(array, any);
      }
  }
  WorkArrayPush.NAME = 'ArrayPush';

  class WorkArrayReduce extends Work {
      doProcess(array, extra) {
          this._outValue = array.reduce((previousValue, currentValue, currentIndex, array) => {
              return this._pipe?.start({ previousValue, currentValue, currentIndex, array, extra })[0];
          });
          return super.doProcess(array, extra);
      }
  }
  WorkArrayReduce.NAME = 'ArrayReduce';

  class WorkArrayReverse extends Work {
      doProcess(array) {
          this._outValue = array.reverse();
          return super.doProcess(array);
      }
  }
  WorkArrayReverse.NAME = 'ArrayReverse';

  class WorkArraySet extends Work {
      doProcess(array, key, value) {
          array[key] = value;
          this._outValue = array;
          return super.doProcess(array, key, value);
      }
  }
  WorkArraySet.NAME = 'ArraySet';

  class WorkArrayShift extends Work {
      doProcess(array) {
          this._outValue = array.shift();
          return super.doProcess(array);
      }
  }
  WorkArrayShift.NAME = 'ArrayShift';

  class WorkArraySlice extends Work {
      doProcess(array, start, end) {
          this._outValue = array.slice(start, end);
          return super.doProcess(array, start, end);
      }
  }
  WorkArraySlice.NAME = 'ArraySlice';

  class WorkArraySort extends Work {
      doProcess(array, extra) {
          if (this._pipe) {
              this._outValue = array.sort((a, b) => {
                  return this._pipe?.start({ a, b, extra })[0];
              });
          }
          else {
              this._outValue = array.sort();
          }
          return super.doProcess(array, extra);
      }
  }
  WorkArraySort.NAME = 'ArraySort';

  class WorkArraySplice extends Work {
      doProcess(array, index, length) {
          this._outValue = array.splice(index, length);
          return super.doProcess(array, index, length);
      }
  }
  WorkArraySplice.NAME = 'ArraySplice';

  class WorkArrayUnShift extends Work {
      doProcess(array, any) {
          array.unshift(any);
          this._outValue = array;
          return super.doProcess(array, any);
      }
  }
  WorkArrayUnShift.NAME = 'ArrayUnShift';

  class WorkArrayValue extends Work {
      doProcess(array, key) {
          const names = key.split('.');
          this._outValue = array;
          for (let name of names) {
              this._outValue = this._outValue[name];
          }
          return super.doProcess(array, key);
      }
  }
  WorkArrayValue.NAME = 'ArrayValue';

  class WorkArray extends Work {
      doProcess(input) {
          this._outValue = input || this._origin.input.split(',') || [];
          return super.doProcess(input);
      }
  }
  WorkArray.NAME = 'Array';

  class WorkBooleanAnd extends Work {
      doProcess(a, b) {
          this._outValue = a && b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanAnd.NAME = 'BooleanAnd';

  class WorkBooleanEqual extends Work {
      doProcess(a, b) {
          this._outValue = a === b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanEqual.NAME = 'BooleanEqual';

  class WorkBooleanGreaterEqualThen extends Work {
      doProcess(a, b) {
          this._outValue = a >= b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanGreaterEqualThen.NAME = 'BooleanGreaterEqualThen';

  class WorkBooleanGreaterThen extends Work {
      doProcess(a, b) {
          this._outValue = a > b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanGreaterThen.NAME = 'BooleanGreaterThen';

  class WorkBooleanLessEqualThen extends Work {
      doProcess(a, b) {
          this._outValue = a <= b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanLessEqualThen.NAME = 'BooleanLessEqualThen';

  class WorkBooleanLessThen extends Work {
      doProcess(a, b) {
          this._outValue = a < b;
          return super.doProcess(a, b);
      }
  }
  WorkBooleanLessThen.NAME = 'BooleanLessThen';

  class WorkBooleanNot extends Work {
      doProcess(a) {
          this._outValue = !a;
          return super.doProcess(a);
      }
  }
  WorkBooleanNot.NAME = 'BooleanNot';

  class WorkBooleanOr extends Work {
      doProcess(a, b) {
          this._outValue = a || b;
          return super.doProcess();
      }
  }
  WorkBooleanOr.NAME = 'BooleanOr';

  class WorkBoolean extends Work {
      doProcess(input) {
          this._outValue = input || this._origin.input === 'false' ? false : this._origin.input.length > 0;
          return super.doProcess(input);
      }
  }
  WorkBoolean.NAME = 'Boolean';

  class WorkDateFormat extends Work {
      doProcess(date, format = 'YYYY-mm-dd') {
          let ret;
          const opt = {
              "Y+": date.getFullYear().toString(),
              "m+": (date.getMonth() + 1).toString(),
              "d+": date.getDate().toString(),
              "H+": date.getHours().toString(),
              "M+": date.getMinutes().toString(),
              "S+": date.getSeconds().toString()
          };
          for (let k in opt) {
              ret = new RegExp("(" + k + ")").exec(format);
              if (ret) {
                  format = format.replace(ret[1], (ret[1].length == 1) ? (opt[k]) : (opt[k].padStart(ret[1].length, "0")));
              }
          }
          this._outValue = format;
          return super.doProcess();
      }
  }
  WorkDateFormat.NAME = 'DateFormat';

  class WorkDateGetDate extends Work {
      doProcess(date) {
          this._outValue = date.getDate();
          return super.doProcess(date);
      }
  }
  WorkDateGetDate.NAME = 'DateGetDate';

  class WorkDateGetDay extends Work {
      doProcess(date) {
          this._outValue = date.getDay();
          return super.doProcess(date);
      }
  }
  WorkDateGetDay.NAME = 'DateGetDay';

  class WorkDateGetFullYear extends Work {
      doProcess(date) {
          this._outValue = date.getFullYear();
          return super.doProcess();
      }
  }
  WorkDateGetFullYear.NAME = 'DateGetFullYear';

  class WorkDateGetHours extends Work {
      doProcess(date) {
          this._outValue = date.getHours();
          return super.doProcess();
      }
  }
  WorkDateGetHours.NAME = 'DateGetHours';

  class WorkDateGetMinutes extends Work {
      doProcess(date) {
          this._outValue = date.getMinutes();
          return super.doProcess();
      }
  }
  WorkDateGetMinutes.NAME = 'DateGetMinutes';

  class WorkDateGetSeconds extends Work {
      doProcess(date) {
          this._outValue = date.getSeconds();
          return super.doProcess();
      }
  }
  WorkDateGetSeconds.NAME = 'DateGetSeconds';

  class WorkDateGetTime extends Work {
      doProcess(date) {
          this._outValue = date.getTime();
          return super.doProcess();
      }
  }
  WorkDateGetTime.NAME = 'DateGetTime';

  class WorkDate extends Work {
      doProcess(input) {
          this._outValue = input || new Date(this._origin.input) || new Date();
          return super.doProcess(input);
      }
  }
  WorkDate.NAME = 'Date';

  class WorkNull extends Work {
      doProcess() {
          this._outValue = null;
          return super.doProcess();
      }
  }
  WorkNull.NAME = 'Null';

  class WorkNumberAbs extends Work {
      doProcess(number) {
          this._outValue = Math.abs(number);
          return super.doProcess(number);
      }
  }
  WorkNumberAbs.NAME = 'NumberAbs';

  class WorkNumberACos extends Work {
      doProcess(number) {
          this._outValue = Math.acos(number);
          return super.doProcess(number);
      }
  }
  WorkNumberACos.NAME = 'NumberACos';

  class WorkNumberASin extends Work {
      doProcess(number) {
          this._outValue = Math.asin(number);
          return super.doProcess(number);
      }
  }
  WorkNumberASin.NAME = 'NumberASin';

  class WorkNumberATan extends Work {
      doProcess(number) {
          this._outValue = Math.atan(Number(number));
          return super.doProcess(number);
      }
  }
  WorkNumberATan.NAME = 'NumberATan';

  class WorkNumberCeil extends Work {
      doProcess(number) {
          this._outValue = Math.ceil(number);
          return super.doProcess(number);
      }
  }
  WorkNumberCeil.NAME = 'NumberCeil';

  class WorkNumberCos extends Work {
      doProcess(number) {
          this._outValue = Math.cos(number);
          return super.doProcess(number);
      }
  }
  WorkNumberCos.NAME = 'NumberCos';

  class WorkNumberDivide extends Work {
      doProcess(numberA, numberB) {
          this._outValue = numberA / numberB;
          return super.doProcess(numberA, numberB);
      }
  }
  WorkNumberDivide.NAME = 'NumberDivide';

  class WorkNumberExp extends Work {
      doProcess(number) {
          this._outValue = Math.exp(number);
          return super.doProcess();
      }
  }
  WorkNumberExp.NAME = 'NumberExp';

  class WorkNumberLimitLoop extends Work {
      doProcess(number, min, max) {
          if (number < min) {
              this._outValue = max;
          }
          if (number > max) {
              this._outValue = min;
          }
          return super.doProcess();
      }
  }
  WorkNumberLimitLoop.NAME = 'NumberLimitLoop';

  class WorkNumberFloor extends Work {
      doProcess(number) {
          this._outValue = Math.floor(number);
          return super.doProcess();
      }
  }
  WorkNumberFloor.NAME = 'NumberFloor';

  class WorkNumberLog extends Work {
      doProcess(number) {
          this._outValue = Math.log(number);
          return super.doProcess();
      }
  }
  WorkNumberLog.NAME = 'NumberLog';

  class WorkNumberMax extends Work {
      doProcess(numberA, numberB) {
          this._outValue = Math.max(numberA, numberB);
          return super.doProcess();
      }
  }
  WorkNumberMax.NAME = 'NumberMax';

  class WorkNumberMin extends Work {
      doProcess(numberA, numberB) {
          this._outValue = Math.min(numberA, numberB);
          return super.doProcess();
      }
  }
  WorkNumberMin.NAME = 'NumberMin';

  class WorkNumberMultiply extends Work {
      doProcess(numberA, numberB) {
          this._outValue = numberA * numberB;
          return super.doProcess();
      }
  }
  WorkNumberMultiply.NAME = 'NumberMultiply';

  class WorkNumberPlus extends Work {
      doProcess(numberA, numberB) {
          this._outValue = numberA + numberB;
          return super.doProcess();
      }
  }
  WorkNumberPlus.NAME = 'NumberPlus';

  class WorkNumberPow extends Work {
      doProcess(numberA, numberB) {
          this._outValue = Math.pow(numberA, numberB);
          return super.doProcess();
      }
  }
  WorkNumberPow.NAME = 'NumberPow';

  class WorkNumberRandom extends Work {
      doProcess() {
          this._outValue = Math.random();
          return super.doProcess();
      }
  }
  WorkNumberRandom.NAME = 'NumberRandom';

  class WorkNumberReduce extends Work {
      doProcess(numberA, numberB) {
          this._outValue = numberA - numberB;
          return super.doProcess();
      }
  }
  WorkNumberReduce.NAME = 'NumberReduce';

  class WorkNumberRound extends Work {
      doProcess(number) {
          this._outValue = Math.round(number);
          return super.doProcess();
      }
  }
  WorkNumberRound.NAME = 'NumberRound';

  class WorkNumberSin extends Work {
      doProcess(number) {
          this._outValue = Math.sin(number);
          return super.doProcess();
      }
  }
  WorkNumberSin.NAME = 'NumberSin';

  class WorkNumberSqrt extends Work {
      doProcess(number) {
          this._outValue = Math.sqrt(number);
          return super.doProcess();
      }
  }
  WorkNumberSqrt.NAME = 'NumberSqrt';

  class WorkNumberTan extends Work {
      doProcess(number) {
          this._outValue = Math.tan(number);
          return super.doProcess();
      }
  }
  WorkNumberTan.NAME = 'NumberTan';

  class WorkNumberToFixed extends Work {
      doProcess(numberA, numberB) {
          this._outValue = numberA.toFixed(numberB);
          return super.doProcess();
      }
  }
  WorkNumberToFixed.NAME = 'NumberToFixed';

  class WorkNumber extends Work {
      doProcess(input) {
          this._outValue = input || Number(this._origin.input) || 0;
          return super.doProcess(input);
      }
  }
  WorkNumber.NAME = 'Number';

  class WorkObjectCallMethod extends Work {
      doProcess(selector, method, params) {
          if (selector) {
              selector[method || this._origin.method](params);
          }
          else {
              this._workflow.elements[this._origin.selector][0][method || this._origin.method](params);
          }
          return super.doProcess(selector, method || this._origin.method, params);
      }
  }
  WorkObjectCallMethod.NAME = 'ObjectCallMethod';

  class WorkObjectForIn extends Work {
      doProcess(object, extra) {
          for (let key in object) {
              this._pipe?.start({ key, value: object[key], extra });
          }
          this._outValue = object;
          return super.doProcess(object, extra);
      }
  }
  WorkObjectForIn.NAME = 'ObjectForIn';

  class WorkObjectHasKey extends Work {
      doProcess(object, key) {
          this._outValue = Object.prototype.hasOwnProperty.call(object, key);
          return super.doProcess(object, key);
      }
  }
  WorkObjectHasKey.NAME = 'ObjectHasKey';

  class WorkObjectKeys extends Work {
      doProcess(object) {
          this._outValue = Object.keys(object);
          return super.doProcess(object);
      }
  }
  WorkObjectKeys.NAME = 'ObjectKeys';

  class WorkObjectMerge extends Work {
      doProcess(objectA, objectB) {
          this._outValue = Object.assign({}, objectA, objectB);
          return super.doProcess(objectA, objectB);
      }
  }
  WorkObjectMerge.NAME = 'ObjectMerge';

  class WorkObjectSet extends Work {
      doProcess(object, key, value) {
          let keys = (key || this._origin.key).split('.');
          let step = object || this._origin.object || {};
          let lastKey;
          while (keys.length > 1) {
              lastKey = keys.shift() || '';
              if (!step[lastKey]) {
                  step[lastKey] = {};
              }
              step = step[lastKey];
          }
          let last = keys.shift() || '';
          step[last] = value || this._origin.value;
          this._outValue = object;
          return super.doProcess(object, key, value);
      }
  }
  WorkObjectSet.NAME = 'ObjectSet';

  class WorkObjectValue extends Work {
      doProcess(object, key) {
          key = key.startsWith('data.item.') ? key.replace('data.item.', 'computedItem.') : key;
          key = key.startsWith('data.') ? key.substr(5) : key;
          const names = key.split('.');
          this._outValue = object;
          for (let name of names) {
              this._outValue = this._outValue[name];
          }
          return super.doProcess(object, key);
      }
  }
  WorkObjectValue.NAME = 'ObjectValue';

  class WorkObjectValues extends Work {
      doProcess(object) {
          this._outValue = Object.values(object);
          return super.doProcess();
      }
  }
  WorkObjectValues.NAME = 'ObjectValues';
  WorkObjectValues.NAME = 'ObjectValues';

  class WorkObject extends Work {
      doProcess(input) {
          this._outValue = input || JSON.parse(this._origin.input) || {};
          return super.doProcess(input);
      }
  }
  WorkObject.NAME = 'Object';

  class WorkStringLetterUpperCase extends Work {
      doProcess(string) {
          this._outValue = (string || this._origin.string).charAt(0).toUpperCase() + (string || this._origin.string).slice(1).toLowerCase();
          return super.doProcess(string);
      }
  }
  WorkStringLetterUpperCase.NAME = 'StringLetterUpperCase';

  class WorkStringCharAt extends Work {
      doProcess(string, index) {
          this._outValue = (string || this._origin.string).charAt(index || this._origin.index);
          return super.doProcess(string);
      }
  }
  WorkStringCharAt.NAME = 'StringCharAt';

  class WorkStringCharCodeAt extends Work {
      doProcess(string, index) {
          this._outValue = (string || this._origin.string).charCodeAt(index || this._origin.index);
          return super.doProcess(string);
      }
  }
  WorkStringCharCodeAt.NAME = 'StringCharCodeAt';

  class WorkStringConcat extends Work {
      doProcess(first, second) {
          this._outValue = (first || this._origin.first).concat(second || this._origin.second);
          return super.doProcess(first, second);
      }
  }
  WorkStringConcat.NAME = 'StringConcat';

  class WorkStringFromCharCode extends Work {
      doProcess(input) {
          this._outValue = String.fromCharCode(input || this._origin.input);
          return super.doProcess(input);
      }
  }
  WorkStringFromCharCode.NAME = 'StringFromCharCode';

  class WorkStringIndexOf extends Work {
      doProcess(stringA, stringB, start) {
          this._outValue = (stringA || this._origin.stringA).indexOf((stringB || this._origin.stringB), Number(start || this._origin.start));
          return super.doProcess(stringA, stringB, start);
      }
  }
  WorkStringIndexOf.NAME = 'StringIndexOf';

  class WorkStringFromJSON extends Work {
      doProcess(string, init = '') {
          try {
              this._outValue = JSON.stringify(string || this._origin.string) || init;
          }
          catch (error) {
              this._outValue = init || '';
          }
          return super.doProcess(string, init);
      }
  }
  WorkStringFromJSON.NAME = 'StringFromJSON';

  class WorkStringToJSON extends Work {
      doProcess(string, init = {}) {
          try {
              this._outValue = JSON.parse(string || this._origin.string) || init;
          }
          catch (error) {
              this._outValue = init || {};
          }
          return super.doProcess(string, init);
      }
  }
  WorkStringToJSON.NAME = 'StringToJSON';

  class WorkStringLastIndexOf extends Work {
      doProcess(stringA, stringB, start) {
          this._outValue = (stringA || this._origin.stringA).lastIndexOf((stringB || this._origin.stringB), Number(start || this._origin.start));
          return super.doProcess(stringA, stringB, start);
      }
  }
  WorkStringLastIndexOf.NAME = 'StringLastIndexOf';

  class WorkStringLength extends Work {
      doProcess(string) {
          this._outValue = (string || this._origin.string).length;
          return super.doProcess(string);
      }
  }
  WorkStringLength.NAME = 'StringLength';

  class WorkStringRegTest extends Work {
      doProcess(regexp, input) {
          this._outValue = new RegExp(regexp || this._origin.regexp).test(input || this._origin.regexp);
          return super.doProcess(regexp, input);
      }
  }
  WorkStringRegTest.NAME = 'StringRegTest';

  class WorkStringReplace extends Work {
      doProcess(string, regexp, replacement) {
          this._outValue = (string || this._origin.string).replace(new RegExp(regexp || this._origin.regexp), (replacement || this._origin.replacement));
          return super.doProcess(string, regexp, replacement);
      }
  }
  WorkStringReplace.NAME = 'StringReplace';

  class WorkStringSlice extends Work {
      doProcess(string, start, end) {
          this._outValue = string.slice(Number(start), Number(end));
          return super.doProcess(string, start, end);
      }
  }
  WorkStringSlice.NAME = 'StringSlice';

  class WorkStringSplit extends Work {
      doProcess(string, separator, length) {
          this._outValue = string.split(separator, length);
          return super.doProcess(string, separator, length);
      }
  }
  WorkStringSplit.NAME = 'StringSplit';

  class WorkStringSubstr extends Work {
      doProcess(string, start, length) {
          this._outValue = string.substr(start, length);
          return super.doProcess(string, start, length);
      }
  }
  WorkStringSubstr.NAME = 'StringSubstr';

  class WorkStringSubstring extends Work {
      doProcess(string, start, end) {
          this._outValue = string.substring(start, end);
          return super.doProcess(string, start, end);
      }
  }
  WorkStringSubstring.NAME = 'StringSubstring';

  class WorkStringToLowerCase extends Work {
      doProcess(string) {
          this._outValue = string.toLowerCase();
          return super.doProcess(string);
      }
  }
  WorkStringToLowerCase.NAME = 'StringToLowerCase';

  class WorkStringToUpperCase extends Work {
      doProcess(string) {
          this._outValue = string.toUpperCase();
          return super.doProcess(string);
      }
  }
  WorkStringToUpperCase.NAME = 'StringToUpperCase';

  class WorkString extends Work {
      doProcess(input) {
          this._outValue = input || this._origin.input || '';
          return super.doProcess(input);
      }
  }
  WorkString.NAME = 'String';

  class WorkEncodeURIComponent extends Work {
      doProcess(input) {
          this._outValue = encodeURIComponent(input);
          return super.doProcess();
      }
  }
  WorkEncodeURIComponent.NAME = 'EncodeURIComponent';

  class WorkMD5 extends Work {
      doProcess(key) {
          this._outValue = null;
          return super.doProcess();
      }
  }
  WorkMD5.NAME = 'MD5';

  class WorkHttpGraphQL extends Work {
  }
  WorkHttpGraphQL.NAME = 'HttpGraphQL';

  class WorkHttpRest extends Work {
  }
  WorkHttpRest.NAME = 'HttpRest';

  var LOC;
  (function (LOC) {
      LOC["href"] = "href";
      LOC["hash"] = "hash";
      LOC["host"] = "host";
      LOC["hostname"] = "hostname";
      LOC["origin"] = "origin";
      LOC["pathname"] = "pathname";
      LOC["port"] = "port";
      LOC["protocol"] = "protocol";
      LOC["search"] = "search";
  })(LOC || (LOC = {}));
  class WorkWindowLocation extends Work {
      doProcess(field) {
          switch (field) {
              case LOC.href:
                  this._outValue = window.location.href;
                  break;
              case LOC.hash:
                  this._outValue = window.location.hash;
                  break;
              case LOC.host:
                  this._outValue = window.location.host;
                  break;
              case LOC.hostname:
                  this._outValue = window.location.hostname;
                  break;
              case LOC.origin:
                  this._outValue = window.location.origin;
                  break;
              case LOC.pathname:
                  this._outValue = window.location.pathname;
                  break;
              case LOC.port:
                  this._outValue = window.location.port;
                  break;
              case LOC.protocol:
                  this._outValue = window.location.protocol;
                  break;
              case LOC.search:
                  this._outValue = window.location.search;
                  break;
              default:
                  this._outValue = window.location;
          }
          return super.doProcess();
      }
  }
  WorkWindowLocation.NAME = 'WindowLocation';

  class WorkLocationNoAnchor extends Work {
      doProcess() {
          this._outValue = window.location.origin + window.location.pathname + window.location.search;
          return super.doProcess();
      }
  }
  WorkLocationNoAnchor.NAME = 'LocationNoAnchor';

  class WorkSocketCreate extends Work {
      doProcess(url, config, extra) {
          const ws = window.io(url, {
              transports: ['websocket']
          });
          ws.on('connect', () => {
              this._pipe?.start({ message: { type: 'open' }, ws, extra });
          });
          ws.on('error', (e) => {
              this._pipe?.start({ message: { type: 'error' }, ws, extra });
          });
          ws.on('disconnect', () => {
              if (!config.close_giveup) {
                  ws.open();
              }
              this._pipe?.start({ message: { type: 'close' }, ws, extra });
          });
          ws.on('message', (e) => {
              lastHeartTime = new Date().getTime();
              if (typeof e.data === 'string' && e.data.startsWith('42')) {
                  let str = e.data.replace('42[', '[');
                  let evt = {
                      type: 'message',
                      data: JSON.parse(str)
                  };
                  this._pipe?.start({ mesage: evt, ws, extra });
              }
          });
          let lastHeartTime = new Date().getTime();
          let ping = () => {
              if (new Date().getTime() - lastHeartTime < 29000) {
                  ws.send(2);
              }
          };
          setInterval(ping, 25000);
          window.ws = ws;
          this._outValue = ws;
          return super.doProcess();
      }
  }
  WorkSocketCreate.NAME = 'SocketCreate';

  class WorkSocketEmit extends Work {
      doProcess(event, string) {
          let ws = window.ws;
          ws.emit(event, string);
          this._outValue = string;
          return super.doProcess();
      }
  }
  WorkSocketEmit.NAME = 'SocketEmit';

  class WorkSocketSend extends Work {
      doProcess(string) {
          let ws = window.ws;
          ws.emit('send', string);
          this._outValue = string;
          return super.doProcess();
      }
  }
  WorkSocketSend.NAME = 'SocketSend';

  class WorkObjectFromQuery extends Work {
      doProcess(input) {
          const obj = {};
          const reg = /([^?&=]+)=([^?&=]*)/g;
          input.replace(reg, function (rs, $1, $2) {
              const name = decodeURIComponent($1);
              let val = decodeURIComponent($2);
              val = String(val);
              obj[name] = val;
              return rs;
          });
          this._outValue = obj;
          return super.doProcess();
      }
  }
  WorkObjectFromQuery.NAME = 'ObjectFromQuery';

  class WorkQueryFromObject extends Work {
      doProcess(input) {
          let query = '';
          for (let key in input) {
              query += key + '=' + input[key] + '&';
          }
          this._outValue = query.slice(0, -1);
          return super.doProcess();
      }
  }
  WorkQueryFromObject.NAME = 'QueryFromObject';

  class WorkQueryURL extends Work {
      doProcess(query) {
          let url = decodeURI(window.location.search);
          let result = url.match(new RegExp('[?&]' + query + '=([^&]+)', 'i'));
          if (result) {
              this._outValue = result[1];
          }
          else {
              url = decodeURI(window.location.hash);
              result = url.match(new RegExp('[?&]' + query + '=([^&]+)', 'i'));
              this._outValue = result ? result[1] : null;
          }
          return super.doProcess();
      }
  }
  WorkQueryURL.NAME = 'QueryURL';

  class Manager {
      constructor() {
          this.map = new Map();
          this.init();
      }
      static getInstance() {
          if (!this.__instance) {
              this.__instance = new Manager();
          }
          return this.__instance;
      }
      set(CLASS) {
          let CLASS_NAME = CLASS.toString();
          CLASS_NAME = CLASS_NAME.slice(9, CLASS_NAME.indexOf('(')).trim();
          if (!CLASS.NAME && CLASS_NAME !== 'Work') {
              throw new Error('need class NAME of ' + CLASS_NAME);
          }
          if (this.map.get(CLASS.NAME)) {
              throw new Error(CLASS.NAME + ' duplication');
          }
          this.map.set(CLASS.NAME, CLASS);
      }
      get(NAME) {
          return this.map.get(NAME);
      }
      build(option, workflow, useLogger) {
          const CLASS = this.get(option.type) || this.get('Worker');
          return new CLASS(option, workflow, useLogger);
      }
      init() {
          this.set(Work);
          const modules = 
    (function() {
      var map = {
        './logic/block.ts': WorkBlock,
  './logic/case.ts': WorkCase,
  './logic/delay.ts': WorkDelay,
  './logic/ifelse.ts': WorkIfElse,
  './logic/switch.ts': WorkSwitch,
  './variable/array/concat.ts': WorkArrayConcat,
  './variable/array/extend/mergebyfield.ts': WorkArrayFieldMerge,
  './variable/array/extend/remove.ts': WorkArrayRemove,
  './variable/array/extend/sum.ts': WorkArraySum,
  './variable/array/extend/sumbyfield.ts': WorkArrayFieldSum,
  './variable/array/filter.ts': WorkArrayFilter,
  './variable/array/flat.ts': WorkArrayFlat,
  './variable/array/foreach.ts': WorkArrayForEach,
  './variable/array/indexof.ts': WorkArrayIndexOf,
  './variable/array/join.ts': WorkArrayJoin,
  './variable/array/lastindexof.ts': WorkArrayLastIndexOf,
  './variable/array/length.ts': WorkArrayLength,
  './variable/array/map.ts': WorkArrayMap,
  './variable/array/pop.ts': WorkArrayPop,
  './variable/array/push.ts': WorkArrayPush,
  './variable/array/reduce.ts': WorkArrayReduce,
  './variable/array/reverse.ts': WorkArrayReverse,
  './variable/array/set.ts': WorkArraySet,
  './variable/array/shift.ts': WorkArrayShift,
  './variable/array/slice.ts': WorkArraySlice,
  './variable/array/sort.ts': WorkArraySort,
  './variable/array/splice.ts': WorkArraySplice,
  './variable/array/unshift.ts': WorkArrayUnShift,
  './variable/array/value.ts': WorkArrayValue,
  './variable/array.ts': WorkArray,
  './variable/boolean/and.ts': WorkBooleanAnd,
  './variable/boolean/equal.ts': WorkBooleanEqual,
  './variable/boolean/greaterequalthen.ts': WorkBooleanGreaterEqualThen,
  './variable/boolean/greaterthen.ts': WorkBooleanGreaterThen,
  './variable/boolean/lessequalthen.ts': WorkBooleanLessEqualThen,
  './variable/boolean/lessthen.ts': WorkBooleanLessThen,
  './variable/boolean/not.ts': WorkBooleanNot,
  './variable/boolean/or.ts': WorkBooleanOr,
  './variable/boolean.ts': WorkBoolean,
  './variable/date/format.ts': WorkDateFormat,
  './variable/date/getdate.ts': WorkDateGetDate,
  './variable/date/getday.ts': WorkDateGetDay,
  './variable/date/getfullyear.ts': WorkDateGetFullYear,
  './variable/date/gethours.ts': WorkDateGetHours,
  './variable/date/getminutes.ts': WorkDateGetMinutes,
  './variable/date/getseconds.ts': WorkDateGetSeconds,
  './variable/date/gettime.ts': WorkDateGetTime,
  './variable/date.ts': WorkDate,
  './variable/null.ts': WorkNull,
  './variable/number/abs.ts': WorkNumberAbs,
  './variable/number/acos.ts': WorkNumberACos,
  './variable/number/asin.ts': WorkNumberASin,
  './variable/number/atan.ts': WorkNumberATan,
  './variable/number/ceil.ts': WorkNumberCeil,
  './variable/number/cos.ts': WorkNumberCos,
  './variable/number/divide.ts': WorkNumberDivide,
  './variable/number/exp.ts': WorkNumberExp,
  './variable/number/extend/limitloop.ts': WorkNumberLimitLoop,
  './variable/number/floor.ts': WorkNumberFloor,
  './variable/number/log.ts': WorkNumberLog,
  './variable/number/max.ts': WorkNumberMax,
  './variable/number/min.ts': WorkNumberMin,
  './variable/number/multiply.ts': WorkNumberMultiply,
  './variable/number/plus.ts': WorkNumberPlus,
  './variable/number/pow.ts': WorkNumberPow,
  './variable/number/random.ts': WorkNumberRandom,
  './variable/number/reduce.ts': WorkNumberReduce,
  './variable/number/round.ts': WorkNumberRound,
  './variable/number/sin.ts': WorkNumberSin,
  './variable/number/sqrt.ts': WorkNumberSqrt,
  './variable/number/tan.ts': WorkNumberTan,
  './variable/number/tofixed.ts': WorkNumberToFixed,
  './variable/number.ts': WorkNumber,
  './variable/object/callmethod.ts': WorkObjectCallMethod,
  './variable/object/forin.ts': WorkObjectForIn,
  './variable/object/haskey.ts': WorkObjectHasKey,
  './variable/object/keys.ts': WorkObjectKeys,
  './variable/object/merge.ts': WorkObjectMerge,
  './variable/object/set.ts': WorkObjectSet,
  './variable/object/value.ts': WorkObjectValue,
  './variable/object/values.ts': WorkObjectValues,
  './variable/object.ts': WorkObject,
  './variable/string/case/letteruppercase.ts': WorkStringLetterUpperCase,
  './variable/string/charat.ts': WorkStringCharAt,
  './variable/string/charcodeat.ts': WorkStringCharCodeAt,
  './variable/string/concat.ts': WorkStringConcat,
  './variable/string/fromcharcode.ts': WorkStringFromCharCode,
  './variable/string/indexof.ts': WorkStringIndexOf,
  './variable/string/json/fromjson.ts': WorkStringFromJSON,
  './variable/string/json/tojson.ts': WorkStringToJSON,
  './variable/string/lastindexof.ts': WorkStringLastIndexOf,
  './variable/string/length.ts': WorkStringLength,
  './variable/string/reg/test.ts': WorkStringRegTest,
  './variable/string/replace.ts': WorkStringReplace,
  './variable/string/slice.ts': WorkStringSlice,
  './variable/string/split.ts': WorkStringSplit,
  './variable/string/substr.ts': WorkStringSubstr,
  './variable/string/substring.ts': WorkStringSubstring,
  './variable/string/tolowercase.ts': WorkStringToLowerCase,
  './variable/string/touppercase.ts': WorkStringToUpperCase,
  './variable/string.ts': WorkString,
  './window/encode/encodeuricomponent.ts': WorkEncodeURIComponent,
  './window/encode/md5.ts': WorkMD5,
  './window/http/graphql.ts': WorkHttpGraphQL,
  './window/http/rest.ts': WorkHttpRest,
  './window/location/location.ts': WorkWindowLocation,
  './window/location/noanchor.ts': WorkLocationNoAnchor,
  './window/socket/create.ts': WorkSocketCreate,
  './window/socket/emit.ts': WorkSocketEmit,
  './window/socket/send.ts': WorkSocketSend,
  './window/url/objectfromquery.ts': WorkObjectFromQuery,
  './window/url/queryfromobject.ts': WorkQueryFromObject,
  './window/url/urlparam.ts': WorkQueryURL,

      };
      var req = function req(key) {
        return map[key] || (function() { throw new Error("Cannot find module '" + key + "'.") }());
      };
      req.keys = function() {
        return Object.keys(map);
      };
      return req;
    })()
  ;
          modules.keys().forEach(key => {
              if (key.split('/').length < 3)
                  return;
              this.set(modules(key));
          });
      }
  }

  class WorkFlow {
      constructor(workOptions, elements, useLogger) {
          this._workQueue = [];
          this._workMap = {};
          this._workOptions = workOptions;
          this._elements = elements;
          this._logger = useLogger(Keys.WORKFLOW);
          workOptions.forEach(workOption => {
              const work = Manager.getInstance().build(workOption, this, useLogger);
              this._workMap[work.id] = work;
          });
          this._workQueue = Object.values(this._workMap).filter(work => !work.outputs.length);
      }
      get logger() {
          return this._logger;
      }
      get elements() {
          return this._elements;
      }
      get current() {
          return this._current || this._workQueue[0];
      }
      set current(value) {
          this._current = value;
      }
      start(data) {
          return this._workQueue.reduce((p, work) => {
              p.push(work?.start(data));
              return p;
          }, []);
      }
      search(id) {
          return this._workMap[id];
      }
  }

  exports.Manager = Manager;
  exports.Work = Work;
  exports.WorkFlow = WorkFlow;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
